#: set fileencoding=utf-8 :
from django.shortcuts import get_object_or_404, render_to_response
from django.http import Http404, HttpResponseRedirect
from django.contrib.auth.decorators import login_required
from django.template import RequestContext
from django.conf import settings
from django.core.urlresolvers import reverse
from django.core.paginator import Paginator, InvalidPage, EmptyPage


from django.utils.translation import ugettext_lazy as _

import calendar
from datetime import datetime,date
from itertools import groupby
import socket, struct

from katrin.models import Client, Period
from traff.models import *
from traff.forms import DetailedStatForm

class TrafficStat:

	class TrafficInfo:
		pass

	def __repr__(self):
		dir(self)

	def __init__(self, the_action, *args, **kwargs):
		if hasattr(self, the_action):
			getattr(self, the_action)(*args, **kwargs)
		else:
			print "action not found"
	
	def get_all_stat(self,start_date, end_date, stat_klass=ServiceTraffStatDay):
		summary = stat_klass.get_stat_summary(start_date,end_date)
		self.download = summary['download']
		self.upload = summary['upload']
		self.download_cost = summary['download_cost']
		self.upload_cost = summary['upload_cost']

		self.get_stat_byfilters(start_date, end_date, stat_klass)
		self.get_timestat(start_date, end_date, stat_klass)

	def get_clients_stat(self, start_date, end_date, stat_klass=ServiceTraffStatDay):
		self.get_stat_byfilters(start_date, end_date, stat_klass,('client__login',))

		""" Get all filters from the stat """
		self.byfilters.sort(key=lambda traff_info: (traff_info['name']))
		self.filters = []

		for filter, i in groupby(self.byfilters, lambda traff_info: traff_info['name']):
			self.filters.append(filter)

		""" Get stat sorted: | 		-> By filters
                             V By clients 
		"""
		self.byfilters.sort(key=lambda traff_info: (traff_info['client__login'], traff_info['name']))
		self.clients_filters = []
		last=None
		for client, i in groupby(self.byfilters, lambda traff_info: traff_info['client__login']):
			filters = list(i)
			client_filters=[]
			for filter in self.filters:
				found=0
				for clf in filters:
					if clf['name']==filter:
						found=1
						client_filters.append(clf)
				if not found:
					client_filters.append({'name':filter,'bytes':0, 'cost':0, 'client__login':client,})
			self.clients_filters.append(client_filters)

	def get_perclient_stat(self, client, start_date, end_date, stat_klass=ServiceTraffStatDay):
		filter_by = {'client__login':client,}
		summary = stat_klass.get_stat_summary(start_date,end_date,**filter_by)
		self.download = summary['download']
		self.upload = summary['upload']
		self.download_cost = summary['download_cost']
		self.upload_cost = summary['upload_cost']

		self.get_stat_byfilters(start_date, end_date, stat_klass, **filter_by)
		self.get_timestat(start_date, end_date, stat_klass, **filter_by)


	def get_stat_byfilters(self,start_date, end_date, stat_klass=ServiceTraffStatDay,add_agg_by=(), **also_filter_by):
		filter_by={'filter__filter_group__isnull': False,}
		filter_by.update(also_filter_by)
		filtergroup_stat = stat_klass.get_aggregated_stat(start_date,end_date, add_agg_by + ('filter__filter_group' ,'filter__filter_group__name',), **filter_by)
		self.byfilters = []
		for fg in filtergroup_stat:
			t = {}
			t['name'] = fg['filter__filter_group__name']
			t['bytes'] = fg['bytes_sum']
			t['cost'] = fg['cost_sum']
			for field in add_agg_by:
				t[field] = fg[field]
			self.byfilters.append(t)

		filter_by={'filter__filter_group__isnull': True,}
		filter_by.update(also_filter_by)
		filter_stat = stat_klass.get_aggregated_stat(start_date,end_date, add_agg_by + ('filter','filter__title', 'filter__filter_group',), **filter_by)
		for f in filter_stat:
			t = {}
			t['name'] = f['filter__title']
			t['bytes'] = f['bytes_sum']
			t['cost'] =f['cost_sum']
			for field in add_agg_by:
				t[field] = f[field]
			self.byfilters.append(t)

	def get_detailed_stat(self,start_period, end_period, fields, also_group_by=(), **also_filter_by):
		def prepare_entry(stat_item, columns):
			entry = {}
			entry['columns'] = []
			for value in columns:
				if stat_item.has_key(value):
					if value=='addr':
						entry['columns'].append(socket.inet_ntoa(struct.pack('>L', stat_item[value])))
					elif value=='in_traff':
						entry['columns'].append(TrafficDirection[stat_item[value]])
					else:
						entry['columns'].append(stat_item[value])
			return entry

		self.detailed_stat = []
		if 'filter' in fields:
			lfields = list(fields)
			i=lfields.index('filter')
			""" We don't need filter for filter group stat """
			lfields.pop(i)
			filter_by={'filter__filter_group__isnull': False,}
			filter_by.update(also_filter_by)
			filtergroup_stat = ServiceTraffStat.get_stat(start_period,end_period, also_group_by + tuple(lfields) + ('filter__filter_group' ,'filter__filter_group__name',), **filter_by).order_by('-bytes_sum')
			lfields.insert(i,'filter__filter_group__name')
			for fg in filtergroup_stat:
				t = prepare_entry(fg, lfields)
				t['bytes'] = fg['bytes_sum']
				t['cost'] = fg['cost_sum']
				self.detailed_stat.append(t)

			""" We don't need filtergroup for filter stat """
			lfields.pop(i)

			filter_by={'filter__filter_group__isnull': True,}
			filter_by.update(also_filter_by)
			filter_stat = ServiceTraffStat.get_stat(start_period,end_period, also_group_by + tuple(lfields) + ('filter','filter__title', 'filter__filter_group',), **filter_by).order_by('-bytes_sum')
			lfields.insert(i,'filter__title')
			for f in filter_stat:
				t = prepare_entry(f, lfields)
				t['cost'] = f['cost_sum']
				t['bytes'] = f['bytes_sum']
				self.detailed_stat.append(t)
		else:
			stat = ServiceTraffStat.get_stat(start_period, end_period, also_group_by + fields, **also_filter_by).order_by('-bytes_sum')
			for item in stat:
				entry = prepare_entry(item,fields)
				entry['bytes'] = item['bytes_sum']
				entry['cost'] = item['cost_sum']
				self.detailed_stat.append(entry)


	def get_timestat(self,start_date, end_date, stat_klass=ServiceTraffStatDay,agg_by=('date_time','in_traff',),  **filter_by):
		timestat = stat_klass.get_aggregated_stat(start_date,end_date,agg_by,**filter_by)
		self.timestat = []
		datetimelast=None
		indexlast = -1
		self.max_input_bytes=0
		self.max_output_bytes=0
		for s in timestat:
			t = self.TrafficInfo()
			t.input = 0
			t.input_cost = 0
			t.output = 0
			t.output_cost = 0
			theentry=None
			if s['date_time'] == datetimelast:
				theentry=self.timestat[indexlast]
			else:
				theentry=t
				t.period = s['date_time']
				indexlast+=1
				self.timestat.append(t)
				datetimelast=s['date_time']

			if s['in_traff']:
				theentry.input = s['bytes_sum']
				theentry.input_cost = s['cost_sum']
				self.max_input_bytes = max(self.max_input_bytes,s['bytes_sum'])
			else:
				theentry.output = s['bytes_sum']
				theentry.output_cost = s['cost_sum']
				self.max_output_bytes = max(self.max_output_bytes,s['bytes_sum'])


@login_required()
def show_menu(request):
	context = {
				'title': _('Traffic stat'),
				'year': date.today().year,
				'month': date.today().month,
				'day': date.today().day,
			}

	return render_to_response('traff/menu.html',
			context,
			context_instance=RequestContext(request))


def all_by_months(request,year):
	year, start_date, end_date = get_year_interval(request, year)

	stat = TrafficStat("get_all_stat", start_date, end_date, ServiceTraffStatMonth)
	return render_to_response('traff/all_by_months.html',
		{
		'title': u'%s %s %s ' % (_('Common stat per'), year, _('year')),
		'year': year,
		'years': get_years(),
		'stat': stat,
		'WITHOUT_COST': settings.WITHOUT_COST
		},
		context_instance=RequestContext(request)
		)

def all_by_days(request,year,month):
	year, month, start_date, end_date = get_year_month_interval(request, year, month)
	stat = TrafficStat("get_all_stat",start_date, end_date)
	return render_to_response('traff/all_by_days.html',
		{
		'title': u'%s %s %s %s %s ' % (_('Common stat per'), month, _('month'), year, _('year')),
		'month': month,
		'months': get_months(),
		'year': year,
		'years': get_years(),
		'stat': stat,
		'WITHOUT_COST': settings.WITHOUT_COST
		},
		context_instance=RequestContext(request)
		)


def all_by_hours(request,year,month,day):
	try:
		year, month, day, start_date, end_date = get_year_month_day_interval(request,year,month,day)
	except ValueError:
		raise Http404

	stat = TrafficStat("get_all_stat",start_date, end_date, ServiceTraffStatHour)

	return render_to_response('traff/all_by_hours.html',
		{
		'title': u'%s %s' % (_('Common stat per'), date(year, month, day )),
		'day': day,
		'days': get_days(),
		'month': month,
		'months': get_months(),
		'year': year,
		'years': get_years(),
		'stat': stat,
		'WITHOUT_COST': settings.WITHOUT_COST
		},
		context_instance=RequestContext(request)
		)

#clients stat
def byclients_per_year(request,year):
	year, start_date, end_date = get_year_interval(request, year)
	stat = TrafficStat("get_clients_stat", start_date, end_date, ServiceTraffStatMonth)
	return render_to_response('traff/byclients_per_year.html',
		{
		'title': u'%s %s %s ' % (_("Clients' stat per"), year, _('year')),
		'year': year,
		'years': get_years(),
		'stat': stat,
		'WITHOUT_COST': settings.WITHOUT_COST
		},
		context_instance=RequestContext(request)
		)

def byclients_per_month(request,year,month):
	year, month, start_date, end_date = get_year_month_interval(request, year, month)
	stat = TrafficStat("get_clients_stat", start_date, end_date)
	return render_to_response('traff/byclients_per_month.html',
		{
		'title': u'%s %s %s %s %s ' % (_("Clients' stat per"), month, _('month'), year, _('year')),
		'month': month,
		'months': get_months(),
		'year': year,
		'years': get_years(),
		'stat': stat,
		'WITHOUT_COST': settings.WITHOUT_COST
		},
		context_instance=RequestContext(request)
		)


def byclients_per_day(request,year,month,day):
	year, month, day, start_date, end_date = get_year_month_day_interval(request,year,month,day)
	try:
		stat = TrafficStat("get_clients_stat", start_date, end_date, ServiceTraffStatHour)
	except ValueError:
		raise Http404

	return render_to_response('traff/byclients_per_day.html',
		{
		'title': u'%s %s' % (_("Clients' stat per"), date(year, month, day) ),
		'day': day,
		'days': get_days(),
		'month': month,
		'months': get_months(),
		'year': year,
		'years': get_years(),
		'stat': stat,
		'WITHOUT_COST': settings.WITHOUT_COST
		},
		context_instance=RequestContext(request)
		)

# per client stat 
	
def perclient_per_year(request, client, year,):
	year, start_date, end_date = get_year_interval(request, year)
	stat = TrafficStat("get_perclient_stat", client, start_date, end_date, ServiceTraffStatMonth)

	return render_to_response('traff/perclient_per_year.html',
		{
		'title': client,
		'uptitle': u'%s %s %s ' % (_("Clients' stat per"), year, _('year')),
		'client': client,
		'year': year,
		'years': get_years(),
		'stat': stat,
		'WITHOUT_COST': settings.WITHOUT_COST
		},
		context_instance=RequestContext(request)
		)

def perclient_per_month(request, client, year, month):
	year, month, start_date, end_date = get_year_month_interval(request, year, month)
	stat = TrafficStat("get_perclient_stat", client, start_date, end_date, ServiceTraffStatDay)
	return render_to_response('traff/perclient_per_month.html',
		{
		'title': client,
		'uptitle': u'%s %s %s %s %s ' % (_("Clients' stat per"), month, _('month'), year, _('year')),
		'client': client,
		'year': year,
		'years': get_years(),
		'month': month,
		'months': get_months(),
		'stat': stat,
		'WITHOUT_COST': settings.WITHOUT_COST
		},
		context_instance=RequestContext(request)
		)

def perclient_per_day(request, client, year, month, day):
	try: 
		year, month, day, start_date, end_date = get_year_month_day_interval(request, year, month, day)
		stat = TrafficStat("get_perclient_stat", client, start_date, end_date, ServiceTraffStatHour)
	except ValueError:
		raise Http404

	return render_to_response('traff/perclient_per_day.html',
		{
		'title': client,
		'uptitle': u'%s %s' % (_("Clients' stat per"), date(year, month, day) ),
		'client': client,
		'year': year,
		'years': get_years(),
		'month': month,
		'months': get_months(),
		'day': day,
		'days': get_days(),
		'stat': stat,
		'WITHOUT_COST': settings.WITHOUT_COST
		},
		context_instance=RequestContext(request)
		)

def detailed_per_client(request,**kwargs):
	if request.method == 'POST':
		form = DetailedStatForm(request.POST)
		if form.is_valid():
			return render_detailed_form(request, form)
	else:
		if 'detailed_stat_form' in request.session.keys():
			form = DetailedStatForm(request.session['detailed_stat_form'],)
			try:
				if form.is_valid():
					return render_detailed_form(request, form)
			except TypeError:
				del request.session['detailed_stat_form']

		form = DetailedStatForm(initial={'columns': ['filter','addr',]},)

	return render_to_response('traff/detailed_per_client.html',
		{
		'title': _('Detailed stat'),
		'form': form,
		'WITHOUT_COST': settings.WITHOUT_COST,
		},
		context_instance=RequestContext(request)
		)

# helpers
def get_years():
	return range(date.today().year-3,date.today().year+1)

def get_months():
	return range(1,13)

def get_days():
	return range(1,32)

def get_year_interval(request, year):
	if (request.POST):
		year = request.POST.get('year', str(date.today().year))
	year = int(year)

	start_date = datetime(year, 1, 1, 0, 0, 0)
	end_date = datetime(year, 12, 1, 0, 0, 0)

	return year, start_date, end_date

def get_year_month_interval(request, year, month):
	if (request.POST):
		year = request.POST.get('year', str(date.today().year))
		month = request.POST.get('month', str(date.today().month))
	year = int(year)
	month = int(month)

	start_date = datetime(year, month, 1, 0, 0, 0)
	end_date = datetime(year, month, calendar.monthrange(year, month)[1], 0, 0, 0)

	return year, month, start_date, end_date

def get_year_month_day_interval(request, year, month, day):
	if (request.POST):
		year = request.POST.get('year', str(date.today().year))
		month = request.POST.get('month', str(date.today().month))
		day = request.POST.get('day', str(date.today().day))

	year = int(year)
	month = int(month)
	day = int(day)

	start_date = datetime(year, month, day, 0, 0, 0)
	end_date = datetime(year, month, day, 23, 0, 0)

	return year, month, day, start_date, end_date

def render_detailed_form(request, form):
	""" Renders detailed stat form. Form must be cleaned """
	cleaned_data = form.cleaned_data

	client = cleaned_data['client']

	start_period = cleaned_data['start']
	end_period = cleaned_data['end']
	column_values = cleaned_data['columns']
	
	data = form.data
	session_data = cleaned_data
	session_data['client'] = data['client'][0]

	request.session['detailed_stat_form'] = session_data
	columns=[]
	for choice in form.fields['columns'].choices:
		if choice[0] in column_values:
			columns.append(choice[1])

	stat = TrafficStat("get_detailed_stat",start_period, end_period, tuple(column_values), client=client)
	stat.detailed_stat.sort(key=lambda x: x['bytes'], reverse=True)
	paginator = Paginator(stat.detailed_stat, settings.LINES_PER_STAT_PAGE)
	# Make sure page request is an int. If not, deliver first page.
	try:
		page = int(request.GET.get('page', '1'))
	except ValueError:
		page = 1

	# If page request (9999) is out of range, deliver last page of results.
	try:
		detailed_stat = paginator.page(page)
	except (EmptyPage, InvalidPage):
		detailed_stat = paginator.page(paginator.num_pages)


	context = {
				'title': _('Detailed traffic stat for %s') % client,
				'WITHOUT_COST': settings.WITHOUT_COST,
				'form': form,
				'columns': columns,
				'detailed_stat': detailed_stat,
			}
	return render_to_response('traff/detailed_per_client.html',
			context,
			context_instance=RequestContext(request))
